/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.freemarker;

import java.net.URL;

/**
 * @author Stian Sigvartsen
 */
public class ClassLoaderTemplateLoader extends URLTemplateLoader {

	public ClassLoaderTemplateLoader(Class loaderClass, String path) {
		if (loaderClass == null) {
			throw new IllegalArgumentException("loaderClass == null");
		}

		if (path == null) {
			throw new IllegalArgumentException("path == null");
		}

		this._loaderClass = loaderClass;
		this._path = canonicalizePrefix(path);
	}

	@Override
	public URL getURL(String name) {
		if (!isTemplateResourceValid(name)) {
			return null;
		}

		return _loaderClass.getResource(_path + name);
	}

	protected static String canonicalizePrefix(String prefix) {
		prefix = prefix.replace('\\', '/');

		if ((prefix.length() > 0) && !prefix.endsWith("/")) {
			prefix += "/";
		}

		return prefix;
	}

	private Class _loaderClass;
	private String _path;

}