/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.freemarker;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.servlet.ServletContextPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.util.PortalUtil;

import java.io.IOException;

import java.net.URL;

import javax.servlet.ServletContext;

/**
 * @author Mika Koivisto
 */
public class ServletTemplateLoader extends URLTemplateLoader {

	public URL getURL(String name) throws IOException {
		URL url = null;

		int pos = name.indexOf(SERVLET_SEPARATOR);

		if (pos != -1) {
			if (!isTemplateResourceValid(name)) {
				return null;
			}

			String servletContextName = name.substring(0, pos);

			if (Validator.isNull(servletContextName)) {
				servletContextName = PortalUtil.getPathContext();
			}

			ServletContext servletContext = ServletContextPool.get(
				servletContextName);

			if (servletContext != null) {
				String templateName =
					name.substring(pos + SERVLET_SEPARATOR.length());

				if (_log.isDebugEnabled()) {
					_log.debug(
						name + " is associated with the servlet context " +
							servletContextName + " " + servletContext);
				}

				url = servletContext.getResource(templateName);

				if ((url == null) &&
					(templateName.endsWith("/init_custom.ftl"))) {

					if (_log.isWarnEnabled()) {
						_log.warn(
							"The template " + name + " should be created");
					}

					String portalServletContextName =
						PortalUtil.getPathContext();

					ServletContext portalServletContext =
						ServletContextPool.get(portalServletContextName);

					url = portalServletContext.getResource(
						"/html/themes/_unstyled/template/init_custom.ftl");
				}
			}
			else {
				_log.error(
					name + " is not valid because " + servletContextName +
						" does not map to a servlet context");
			}
		}

		return url;
	}

	private static Log _log = LogFactoryUtil.getLog(
		ServletTemplateLoader.class);

}