/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.servlet.filters.util;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.model.ColorScheme;
import com.liferay.portal.model.Theme;
import com.liferay.portal.service.ThemeLocalServiceUtil;
import com.liferay.portal.util.PortalUtil;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

/**
 * @author José Ángel Jiménez Campoy
 */
public class ColorSchemeIdCacheFileNameContributor
	implements CacheFileNameContributor {

	public String getParameterName() {
		return "colorSchemeId";
	}

	public String getParameterValue(HttpServletRequest request) {
		String themeId = request.getParameter("themeId");

		if (themeId != null) {
			List<Theme> themes = ThemeLocalServiceUtil.getThemes(
				PortalUtil.getCompanyId(request));

			try {
				for (Theme theme: themes) {
					if (!themeId.equals(theme.getThemeId())) {
						continue;
					}

					ColorScheme colorScheme =
						ThemeLocalServiceUtil.getColorScheme(
							PortalUtil.getCompanyId(request),
							themeId, request.getParameter(getParameterName()),
							theme.isWapTheme());

					if (colorScheme != null) {
						return colorScheme.getColorSchemeId();
					}
				}
			}
			catch (SystemException se) {
				return null;
			}
		}

		return null;
	}

}